/*
	* Copyright (C) GTA:Multiplayer Team (https://wiki.gta-mp.net/index.php/Team)
	*
	* Redistribution and use in source and binary forms, with or without
	* modification, are permitted provided that the following conditions are
	* met:
	*
	*     * Redistributions of source code must retain the above copyright
	*		notice, this list of conditions and the following disclaimer.
	*     * Redistributions in binary form must reproduce the above
	*		copyright notice, this list of conditions and the following disclaimer
	*		in the documentation and/or other materials provided with the
	*		distribution.
	*     * Neither the name of GTA-Network nor the names of its
	*		contributors may be used to endorse or promote products derived from
	*		this software without specific prior written permission.
	*
	* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT
	* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	* DATA, OR PROFITS; OR BUSINESS INTERRUPTION HOWEVER CAUSED AND ON ANY
	* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	* INCLUDING NEGLIGENCE OR OTHERWISE ARISING IN ANY WAY OUT OF THE USE
	* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <cstdint>
#include <WinSock2.h>
#include <Windows.h>
#include <string>
#include <vector>
#include <cstdio>
#include <Psapi.h>
#include <cfgparser.h>

// ScriptHookV
#include "scripthookv/main.h"
#include "scripthookv/natives.h"
#pragma comment(lib, "lib/scripthookv.lib")

static bool configEnableDefault = true;
static bool configVehicleTracks = true;
static bool configPedTracks = true;
static bool configVehicleTrackDepth = false;
static bool configPedTrackDepth = false;
static int32_t toggleKey = 48; // KEY 0 (NORMAL KEYBOARD)

// Find Pattern
bool CompareMemory(const uint8_t* pData, const uint8_t* bMask, const char* sMask)
{
	for (; *sMask; ++sMask, ++pData, ++bMask)
		if (*sMask == 'x' && *pData != *bMask)
			return false;

	return *sMask == NULL;
}

intptr_t FindPattern(const char* bMask, const char* sMask)
{
	// Game Base & Size
	static intptr_t pGameBase = (intptr_t) GetModuleHandle(nullptr);
	static uint32_t pGameSize = 0;
	if (!pGameSize)
	{
		MODULEINFO info;
		GetModuleInformation(GetCurrentProcess(), (HMODULE)pGameBase, &info, sizeof(MODULEINFO));
		pGameSize = info.SizeOfImage;
	}

	// Scan
	for (uint32_t i = 0; i < pGameSize; i++)
		if (CompareMemory((uint8_t*)(pGameBase + i), (uint8_t*)bMask, sMask))
			return pGameBase + i;

	return 0;
}

// Map Notification
void PushMapNotification(char* msg)
{
	UI::_SET_NOTIFICATION_TEXT_ENTRY("STRING");
	UI::_ADD_TEXT_COMPONENT_STRING(msg);
	UI::_DRAW_NOTIFICATION(FALSE, FALSE);
}

// Tracks
void EnableTracks(bool tracksVehicle=false, bool tracksPeds=false, bool deepTracksVehicle=false, bool deepTracksPed=false)
{
	static auto VAR_FeetSnowTracks_call = FindPattern("\x80\x3D\x00\x00\x00\x00\x00\x48\x8B\xD9\x74\x37", "xx?????xxxxx");
	if(!VAR_FeetSnowTracks_call)
	{
		PushMapNotification("Snow Mod is ~r~not compatible ~s~with this GTA Version #1!");
		return;
	}
	static auto VAR_FeetSnowTracks = VAR_FeetSnowTracks_call + (*(int32_t *)(VAR_FeetSnowTracks_call + 2)) + 7;
	//
	static auto VAR_VehicleSnowTracks_call = FindPattern("\x40\x38\x3D\x00\x00\x00\x00\x48\x8B\x42\x20", "xxx????xxxx");
	if(!VAR_VehicleSnowTracks_call)
	{
		PushMapNotification("Snow Mod is ~r~not compatible ~s~with this GTA Version #2!");
		return;
	}
	static auto VAR_VehicleSnowTracks = VAR_VehicleSnowTracks_call+ (*(int32_t *)(VAR_VehicleSnowTracks_call + 3)) + 7;
	//

	VirtualProtect((void*) VAR_FeetSnowTracks, 1, PAGE_EXECUTE_READWRITE, nullptr);
	VirtualProtect((void*) VAR_VehicleSnowTracks, 1, PAGE_EXECUTE_READWRITE, nullptr);

	// Enable/Disable Vehicle/Foot Snow tracks
	*(uint8_t *) VAR_FeetSnowTracks = tracksVehicle;
	*(uint8_t *) VAR_VehicleSnowTracks = tracksPeds;

	// Switch for big/small tracks
	static auto vehicleTrackTypes = FindPattern("\xB9\x00\x00\x00\x00\x84\xC0\x44\x0F\x44\xF1", "x????xxxxxx");
	if(!vehicleTrackTypes)
	{
		PushMapNotification("Snow Mod is ~r~not compatible ~s~with this GTA Version #3!");
		return;
	}

	VirtualProtect((void*) vehicleTrackTypes, 1, PAGE_EXECUTE_READWRITE, nullptr);
	*(uint8_t *) (vehicleTrackTypes + 1) = deepTracksVehicle ? 0x13 : 0x14;

	static auto pedTrackTypes = FindPattern("\xB9\x00\x00\x00\x00\x84\xC0\x0F\x44\xD9\x48\x8B\x4F\x30", "x????xxxxxxxxx");
	if(!pedTrackTypes)
	{
		PushMapNotification("Snow Mod is ~r~not compatible ~s~with this GTA Version #4!");
		return;
	}
	VirtualProtect((void*) pedTrackTypes, 1, PAGE_EXECUTE_READWRITE, nullptr);
	*(uint8_t *) (pedTrackTypes + 1) = deepTracksPed ? 0x13 : 0x14;
}

// Snow
void EnableSnow(bool bEnable)
{
	if(bEnable)
		EnableTracks(configVehicleTracks, configPedTracks, configVehicleTrackDepth, configPedTrackDepth);
	else
		EnableTracks();

	// Adresses
	static auto addr1 = FindPattern("\x80\x3D\x00\x00\x00\x00\x00\x74\x27\x84\xC0", "xx?????xxxx");
	static auto addr2 = FindPattern("\x44\x38\x3D\x00\x00\x00\x00\x74\x0F", "xxx????xx");

	// Outdated
	// In future the patterns might change
	if (!addr1 || !addr2)
	{
		PushMapNotification("Snow Mod is ~r~not compatible ~s~with this GTA Version #5!");
		return;
	}

	// Original Memory
	static uint8_t original1[14] = { 0 };
	static uint8_t original2[15] = { 0 };

	// Initialize
	static bool bInitialized = false;
	if (!bInitialized)
	{
		bInitialized = true;

		// Unprotect Memory
		VirtualProtect((void*)addr1, 13, PAGE_EXECUTE_READWRITE, nullptr);
		VirtualProtect((void*)addr2, 14, PAGE_EXECUTE_READWRITE, nullptr);

		// Copy original Memory
		memcpy(&original1, (void*)addr1, 13);
		memcpy(&original2, (void*)addr2, 14);
	}

	// Toggle
	if (bEnable)
	{
		// Weather
		GAMEPLAY::SET_WEATHER_TYPE_NOW_PERSIST("XMAS");

		// NOP checks
		memset((void*)addr1, 0x90, 13);
		memset((void*)addr2, 0x90, 14);

		// Notification
		PushMapNotification("Snow ~g~ON");
	}
	else
	{
		// Copy original memory
		memcpy((void*)addr1, &original1, 13);
		memcpy((void*)addr2, &original2, 14);

		// Weather
		GAMEPLAY::CLEAR_WEATHER_TYPE_PERSIST();
		GAMEPLAY::SET_WEATHER_TYPE_NOW("CLEAR");

		// Notification
		PushMapNotification("Snow ~r~OFF");
	}
}

// Main
void ScriptMain()
{
	static bool bSnowEnabled = false;

	// Config
	static bool bConfigParsed = false;

	if (!bConfigParsed)
	{
		static CFGParser parser("snow.cfg");
		parser.Parse();

		if (parser.KeyExists("enableDefault"))
			configEnableDefault = static_cast<char>(atoi(parser.GetValueOfKey<std::string>("enableDefault").c_str()) != 0);

		if (parser.KeyExists("vehicleTracks"))
			configVehicleTracks = static_cast<char>(atoi(parser.GetValueOfKey<std::string>("vehicleTracks").c_str())) != 0;

		if (parser.KeyExists("pedTracks"))
			configPedTracks = static_cast<char>(atoi(parser.GetValueOfKey<std::string>("pedTracks").c_str())) != 0;

		if (parser.KeyExists("vehicleTrackDepth"))
			configVehicleTrackDepth = static_cast<char>(atoi(parser.GetValueOfKey<std::string>("vehicleTrackDepth").c_str())) != 0;

		if (parser.KeyExists("pedTrackDepth"))
			configPedTrackDepth = static_cast<char>(atoi(parser.GetValueOfKey<std::string>("pedTrackDepth").c_str())) != 0;

		if (parser.KeyExists("toggleKey"))
			toggleKey = static_cast<int32_t>(atoi(parser.GetValueOfKey<std::string>("toggleKey").c_str()));
	
		bConfigParsed = true;  
	}

	// Default
	if (configEnableDefault)
	{
		bSnowEnabled = true;
		EnableSnow(true);
	}

	// Process
	for (;;)
	{
		if(GetAsyncKeyState(toggleKey) & 1)
		{
			bSnowEnabled = !bSnowEnabled;
			EnableSnow(bSnowEnabled);
		}

		WAIT(1);
	}
}

// Main
BOOL WINAPI DllMain(HMODULE hModule, DWORD dwReason, LPVOID pReserved)
{
	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:
		// Register
		scriptRegister(hModule, ScriptMain);
		break;
	case DLL_PROCESS_DETACH:
		// Unregister
		scriptUnregister(ScriptMain);
		break;
	}

	return TRUE;
}