/*
	* Copyright (C) GTA:Multiplayer Team (https://wiki.gta-mp.net/index.php/Team)
	*
	* Redistribution and use in source and binary forms, with or without
	* modification, are permitted provided that the following conditions are
	* met:
	*
	*     * Redistributions of source code must retain the above copyright
	*		notice, this list of conditions and the following disclaimer.
	*     * Redistributions in binary form must reproduce the above
	*		copyright notice, this list of conditions and the following disclaimer
	*		in the documentation and/or other materials provided with the
	*		distribution.
	*     * Neither the name of GTA-Network nor the names of its
	*		contributors may be used to endorse or promote products derived from
	*		this software without specific prior written permission.
	*
	* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
	* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
	* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
	* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
	* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT
	* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	* DATA, OR PROFITS; OR BUSINESS INTERRUPTION HOWEVER CAUSED AND ON ANY
	* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	* INCLUDING NEGLIGENCE OR OTHERWISE ARISING IN ANY WAY OUT OF THE USE
	* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#pragma once

#include <Windows.h>
#include <cfgparser.h>

const char default_config[] =
"--Should the mod be enabled by default?\n"
"enableDefault=1\n"
"--\n"
"--Should vehicle tracks be drawn when snow is active?\n"
"vehicleTracks=1\n"
"--\n"
"--Should pedTracks be drawn when snow is active ?\n"
"pedTracks=1\n"
"--\n"
"--vehicleTrackDepth : 0 means light snow depth, 1 heavy depth\n"
"vehicleTrackDepth=1\n"
"--\n"
"--vehicleTrackDepth : 0 means light snow depth, 1 heavy depth\n"
"pedTrackDepth=1\n"
"--\n"
"--toggle key : you can change that on your own, default is NUMBER 0\n"
"--https://msdn.microsoft.com/en-us/library/windows/desktop/dd375731%28v=vs.85%29.aspx?f=255&MSPPError=-2147217396\n"
"--Please note that the keys there are in hexadecimal format, you have to convert it to decimal!\n"
"--You can use this converter : http://www.binaryhexconverter.com/hex-to-decimal-converter\n"
"toggleKey=48";

CFGParser::CFGParser(const std::string& strName)
{
	this->strName.append(strName.c_str());
}

void CFGParser::Parse()
{
	char szExePath[MAX_PATH] = { 0 };
	GetModuleFileNameA(GetModuleHandle(NULL), szExePath, MAX_PATH);

	// Fix path in string
	for(size_t i = strlen(szExePath); i > 0; --i)
	{
		if(szExePath[i] == '\\')
		{
			szExePath[i + 1] = '\0';
			break;
		}
	}

	std::string strAbsolutePath;
	strAbsolutePath.append(szExePath);
	strAbsolutePath.append(this->strName);
	this->strName = strAbsolutePath;
	this->ExtractKeys();
}

bool CFGParser::KeyExists(const std::string& strKey)
{
	return this->mapContents.find(strKey) != this->mapContents.end();
}

void CFGParser::RemoveComment(std::string& strLine)
{
	if(strLine.find(';') != strLine.npos)
		strLine.erase(strLine.find(';'));
}

bool CFGParser::OnlyWhitespace(const std::string& strLine)
{
	return (strLine.find_first_not_of(' ') == strLine.npos);
}

bool CFGParser::ValidLine(const std::string& strLine)
{
	std::string strTemp = strLine;
	strTemp.erase(0, strTemp.find_first_not_of("\t "));
	if(strTemp[0] == '=')
		return false;

	for(size_t i = strTemp.find('=') + 1; i < strTemp.length(); i++)
	if(strTemp[i] != ' ')
		return true;

	return false;
}

void CFGParser::ExtractKey(std::string& strKey, size_t const& sepPos, const std::string& strLine)
{
	strKey = strLine.substr(0, sepPos);
	if(strKey.find('\t') != strLine.npos || strKey.find(' ') != strLine.npos)
		strKey.erase(strKey.find_first_of("\t "));
}

void CFGParser::ExtractValue(std::string& strValue, size_t const &sepPos, const std::string& strLine)
{
	strValue = strLine.substr(sepPos + 1);
	strValue.erase(0, strValue.find_first_not_of("\t "));
	strValue.erase(strValue.find_last_not_of("\t ") + 1);
}

void CFGParser::ExtractContents(const std::string& strLine)
{
	std::string strTemp = strLine;
	strTemp.erase(0, strTemp.find_first_not_of("\t "));
	size_t sepPos = strTemp.find('=');

	std::string strKey, strValue;
	this->ExtractKey(strKey, sepPos, strTemp);
	this->ExtractValue(strValue, sepPos, strTemp);

	if(strKey.find("--") != strKey.npos)
		return;

	if(!this->KeyExists(strKey))
		this->mapContents.insert(std::pair<std::string, std::string>(strKey, strValue));
	else
		return;
}

void CFGParser::ParseLine(const std::string& strLine, size_t const lineNo)
{
	if(strLine.find('=') == strLine.npos && strLine.find("--") == strLine.npos)
		return;

	if(!this->ValidLine(strLine))
		return;

	this->ExtractContents(strLine);
}

bool CFGParser::DoesFileExist()
{
	std::ifstream pFile;
	pFile.open(this->strName.c_str());
	if(!pFile)
		return false;
	else
		return true;
}

void CFGParser::ExtractKeys()
{
	std::ifstream pFile;
	pFile.open(this->strName.c_str());
	if(!pFile)
	{
		FILE* file = nullptr;
		fopen_s(&file, "snow.cfg", "w");
		if(!file)
			return;

		fwrite(default_config, sizeof(char), sizeof(default_config)-1, file);
		fclose(file);

		ExtractKeys();
	}

	std::string strLine;
	size_t lineNo = 0;
	while(std::getline(pFile, strLine))
	{
		lineNo++;
		std::string strTemp = strLine;

		if(strTemp.empty())
			continue;

		this->RemoveComment(strTemp);
		if(this->OnlyWhitespace(strTemp))
			continue;

		this->ParseLine(strTemp, lineNo);
	}

	pFile.close();
}